#!/usr/bin/env python

# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license.
# Thank you for using free software!
#
#(c) Whise 2010 <helderfraga@gmail.com>
#
# Icon factory
# Part of the GnoMenu

import gtk
import os
import xml.dom.minidom
import gobject
import utils
import Globals
import gc
import urllib
import xdg.IconTheme
try:
	import gio
	isgio = True
except:
	print 'gio not found'
	isgio = False

Icontype = ["png", "xpm", "svg"]

def GetSystemIcon(icon):
	for n in Icontype:
		icon = str(icon).replace('.' + n,'')
	ico = Globals.GtkIconTheme.lookup_icon(icon,48,gtk.ICON_LOOKUP_FORCE_SVG)
	if ico:
		ico = ico.get_filename()
	else:
		ico = None
	return ico

class IconFactory(gobject.GObject):
	__gsignals__ = {"icons-changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [gobject.TYPE_STRING])}

	def __init__(self):
		gobject.GObject.__init__(self)

		##### This loads the icon cache

		self.IconCachePath = Globals.ConfigDirectory + "/.menuiconcache.xml"
		self.IconNames = []
		self.Icons = []
		self.IconPix = {"":""}
		if Globals.Settings['Show_Thumb']:
			self.thumbnailer = utils.thumbnailengine(Globals.PG_iconsize)
			self.thumbnailer.connect("worklist-finished", lambda m: self.emit('icons-changed'))

		##### looksup icon theme in gtk, we could also use : gconf.client_get_default().get_string('/desktop/gnome/interface/icon_theme')

		self.gtkicontheme = gtk.icon_theme_get_default()
		self.icontheme = Globals.DefaultIconTheme
		self.old_icontheme = self.icontheme

		##### Check if icon theme is stored in cache#################
		self.LoadIconCache()


	def LoadIconCache(self):
		print 'loading icon cache'
		if os.path.isfile(self.IconCachePath):
			try:
				XMLSettings = xml.dom.minidom.parse(self.IconCachePath)
				self.cacheicontheme = XMLSettings.getElementsByTagName("cache")[0].attributes["IconTheme"].value
			except:
				print "Recoverable Error: Loading of icon cache has failed! This could be due to a corrupted file or read permissions."
				self.cacheicontheme = None
				os.system('rm ' +Globals.ConfigDirectory + "/.menuiconcache.xml")
			if  self.cacheicontheme == self.icontheme:
				print 'icon theme in cache'
				XBase = XMLSettings.childNodes[0].getElementsByTagName("Icon")
				for node in XBase:
					iconname = node.attributes["IconNames"].value
					self.IconNames.append(iconname)
					icon = node.attributes["Icons"].value
					self.Icons.append(icon)
					if os.path.exists(icon):
						pix = gtk.gdk.pixbuf_new_from_file_at_size(icon, Globals.PG_iconsize, Globals.PG_iconsize)
					else:
						pix =	self.geticonfile(gtk.STOCK_MISSING_IMAGE)
					self.IconPix[iconname] = pix
			else:
				print 'icon theme not in cache - restarting cache'
				os.system('rm ' +Globals.ConfigDirectory + "/.menuiconcache.xml")


	def Icon_change(self):
		"""Icons have changed"""

		print 'icons changed'
		self.icontheme = gtk.settings_get_default().get_property("gtk-icon-theme-name")
		self.gtkicontheme = gtk.icon_theme_get_default()
		if self.icontheme != self.old_icontheme:
				
			self.IconNames = []
			self.Icons = []
			self.IconPix.clear()
			self.old_icontheme = self.icontheme	



	def getgicon(self,gico):
		"""Returns gio icon"""
		try:
			names = self.gtkicontheme.lookup_by_gicon(gico,Globals.PG_iconsize,gtk.ICON_LOOKUP_FORCE_SVG);
			if names:
				return names.get_filename()
		except:pass
		try:
			names = gico.get_names()
			for x in range(0,len(names)):
				if self.gtkicontheme.has_icon(names[x]):
					return names[x]
		except:pass
		return 'gtk-execute'

	def addicontomen(self,icon,iconpath,pix):
		"""Adds icons to cache"""
		self.IconNames.append(icon)
		self.Icons.append(iconpath)
		self.IconPix[icon] = pix
		gc.collect()
		

	def getthumb(self,item):
		path = urllib.url2pathname(item.get_uri().replace('file://',''))
		if Globals.Settings['Show_Thumb']:
			if not os.path.exists(path): 
				return item.get_icon(Globals.PG_iconsize)
			if isgio:
				self.gfile = gio.File(path)
				self.info = self.gfile.query_info(gio.FILE_ATTRIBUTE_THUMBNAIL_PATH, gio.FILE_QUERY_INFO_NONE)
				thumbfile = self.info.get_attribute_as_string(gio.FILE_ATTRIBUTE_THUMBNAIL_PATH)
				if thumbfile:
					if os.path.isfile(thumbfile): return thumbfile
					else:return item.get_icon(Globals.PG_iconsize)
				else:return item.get_icon(Globals.PG_iconsize)



			else:
				icon, isthumbnail = self.thumbnailer.CheckWorkList(path)
				if not icon:
					self.thumbnailer.AddToWorkList(path, item.get_mime_type())
					return item.get_icon(Globals.PG_iconsize)
				else:
					if isthumbnail:
						return icon
					else: return item.get_icon(Globals.PG_iconsize)
		else:
			return item.get_icon(Globals.PG_iconsize)



	def geticonfile(self,icon):

		"""Returns system icon paths"""

		# If icons is pixbuf we do nothing
		if isinstance(icon, gtk.gdk.Pixbuf):
		#if type(icon).__name__ == 'Pixbuf':
			return icon

		# Reload icons if icons have changed since last restart
		if self.icontheme != self.old_icontheme:
				
			self.IconNames = []
			self.Icons = []
			self.IconPix.clear()
			self.old_icontheme = self.icontheme	

		# If icon is in memory return icons, else get icon

		if icon in self.IconNames:

			return self.IconPix[icon]#gtk.gdk.pixbuf_new_from_file_at_size(icon, Globals.PG_iconsize, Globals.PG_iconsize)
		else:

			# lockup icon in gtk icon theme
			
			self.iconinfo = self.gtkicontheme.lookup_icon(str(icon),Globals.PG_iconsize,gtk.ICON_LOOKUP_FORCE_SVG);
			
			if self.iconinfo:
				iconpath = self.iconinfo.get_filename()
				if iconpath:
					if os.path.isfile(iconpath):
						pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)
						self.addicontomen(icon,iconpath,pix)
						gc.collect()
						return pix
				else:
					pix = self.iconinfo.get_builtin_pixbuf()
					if pix:
						return pix
			
			# lockup icon in xdg icon theme
			try:
				iconpath = xdg.IconTheme.getIconPath(icon, Globals.PG_iconsize, self.icontheme, Icontype)
			except: iconpath = None # above receives errors sometimes
			if iconpath:
				if os.path.exists(iconpath):
					pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)	
					self.addicontomen(icon,iconpath,pix)
					gc.collect()
					return pix

			# if icons not found return gtk missing image
			self.iconinfo = self.gtkicontheme.lookup_icon(gtk.STOCK_MISSING_IMAGE,Globals.PG_iconsize,gtk.ICON_LOOKUP_FORCE_SVG);
			if self.iconinfo:
				iconpath = self.iconinfo.get_filename()
				if iconpath:
					if os.path.exists(iconpath):
						pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)
						self.addicontomen(icon,iconpath,pix)
	
						gc.collect()
						return pix
				else:
					pix = self.iconinfo.get_builtin_pixbuf()
					if pix:
						return pix

			# if all else fails return gnomenu own missing image, should never append...

			iconpath = Globals.BrokenImage
			pix = gtk.gdk.pixbuf_new_from_file_at_size(Globals.BrokenImage, Globals.PG_iconsize, Globals.PG_iconsize)
			self.addicontomen(icon,iconpath,pix)
			gc.collect()
			return pix



	def SaveIconCache(self):
		print "Saving Icon Cache..."
		#Save Imagecache
		if os.path.exists(self.IconCachePath):
			f = open(self.IconCachePath,"r").read()
		else: f = None
		try:
			XMLSettings = xml.dom.minidom.Document()
			XBase = XMLSettings.createElement("cache")
			XBase.setAttribute("IconTheme", self.icontheme)
			XMLSettings.appendChild(XBase)
			for icon in range(0,len(self.IconNames)):
				node = XMLSettings.createElement("Icon")
				node.setAttribute("IconNames", self.IconNames[icon])
				node.setAttribute("Icons", self.Icons[icon])
				XBase.appendChild(node)
			#Now write it to the file
			file = open(self.IconCachePath,"w")
			XMLSettings.writexml(file, "    ", "", "\n", "UTF-8")
			XMLSettings.unlink()
			print "Icon cache successfully written to user's home directory"		
		except:
			print "Error - Icon cache saving error"
			if f:
				file = open(self.IconCachePath,"w")
				file.write(f)
				file.close()
